@@ -13,7 +13,7 @@ import json
 from youtube_search_requests.constants import USER_AGENT_HEADERS
 from youtube_search_requests.utils import *
 
-__VERSION__ = 'v0.0.15.5'
+__VERSION__ = 'v0.0.16'
 
 try:
     import youtube_dl
@@ -29,14 +29,15 @@ class YoutubeSearch:
         validate=True,
         timeout=None,
         extract_info=False,
-        json_results=False
+        json_results=False,
+        include_related_videos=False,
     ):
         """
         YoutubeSearch arguments
 
         search_query: :class:`str`
             a string terms want to search
-        max_results: :class:`int`
+        max_results: :class:`int` (optional, default: 10)
             maximum search results
         validate: :class:`bool` (optional, default: True)
             validate url results, validating urls takes too much times but it worth to prevent UNPLAYABLE or ERROR videos
@@ -46,6 +47,8 @@ class YoutubeSearch:
             Extract additional info in urls, NOTE: you need to install youtube-dl module to extract additional info (pip install youtube-dl)
         json_results: :class:`bool` (optional, default: False)
             if True, Return results in json format. If False return results in dict format
+        include_related_videos: :class:`bool` (optional, default: False)
+            include all related videos each url's
         """
 
         # Validate the arguments
@@ -62,14 +65,19 @@ class YoutubeSearch:
             raise InvalidArgument('extract_info expecting bool, got %s' % (extract_info.__class__.__name__))
         if not isinstance(json_results, bool):
             raise InvalidArgument('json_results expecting bool, got %s' % (json_results.__class__.__name__))
+        if not isinstance(include_related_videos, bool):
+            raise InvalidArgument('include_related_videos expecting bool, got %s' % (include_related_videos.__class__.__name__))
 
         self.search_query = search_query
         self.max_results = max_results
         self.BASE_URL = 'https://www.youtube.com/results?search_query='
         self.validate = validate
         self.timeout = timeout
-        self.extract = extract_info
+        self.extract_info = extract_info
         self.json_results = json_results
+        self.include_related_videos = include_related_videos
+        # wait event shutdown worker
+        self._wait_event = None
 
         # Each headers give different results
         # this give advantages to query more results
@@ -121,10 +129,48 @@ class YoutubeSearch:
         try:
             return info['thumbnails']
         except KeyError:
-            return info['thumbnail']
+            return [info['thumbnail']]
         else:
             return None
 
+    def _extract_related_videos(self, url):
+        """
+        extract all related videos in given url
+
+        NOTE: with validate, extract_info options enabled, the search can be really-really slow
+
+        """
+        if not self.include_related_videos:
+            return None
+        legit_urls = []
+        r = requests.get(url)
+        urls = self._parse_urls(r)
+        for i in urls:
+            if i == url:
+                continue
+            if self.validate:
+                if self.validate_url(i):
+                    pass
+                else:
+                    continue
+            else:
+                pass
+            if self.extract_info:
+                if import_youtube_dl:
+                    try:
+                        y = youtube_dl.YoutubeDL(params={'quiet': True, 'no_warnings': True})
+                        info = y.extract_info(i, download=False, process=False)
+                        legit_urls.append({'title': info['title'], 'url': i, 'author': info['uploader'], 'thumbnails': self._extract_thumbnail(info)})
+                    except youtube_dl.utils.DownloadError:
+                        legit_urls.append({'title': None, 'url': i, 'author': None, 'thumbnails': None})
+                else:
+                    raise ModuleNotFoundError('youtube-dl module not found, please install it')
+            else:
+                legit_urls.append(i)
+        return legit_urls
+        
+        
+
     def _extract_info(self, url: str):
         """
         if module youtube-dl is imported.
@@ -132,20 +178,23 @@ class YoutubeSearch:
 
         if not, just return the url.
         """
-        if self.extract:
+        if self.extract_info:
             if import_youtube_dl:
                 try:
                     y = youtube_dl.YoutubeDL(params={'quiet': True, 'no_warnings': True})
                     info = y.extract_info(url, download=False, process=False)
-                    return {'title': info['title'], 'url': url, 'author': info['uploader'], 'thumbnails': self._extract_thumbnail(info)}
+                    return {'title': info['title'], 'url': url, 'author': info['uploader'], 'thumbnails': self._extract_thumbnail(info), 'related_videos': self._extract_related_videos(url)}
                 except youtube_dl.utils.DownloadError:
-                    return {'title': None, 'url': url, 'author': None, 'thumbnails': None}
+                    return {'title': None, 'url': url, 'author': None, 'thumbnails': None, 'related_videos': None}
             else:
                 raise ModuleNotFoundError('youtube-dl module not found, please install it')
         else:
-            return url
+            if self.include_related_videos:
+                return {'url': url, 'related_videos': self._extract_related_videos(url)}
+            else:
+                return url
 
-    def _wrap_json(self, urls):
+    def _wrap_json(self, urls: list):
         if self.json_results:
             return json.dumps({'urls': urls})
         else:
@@ -154,7 +203,7 @@ class YoutubeSearch:
     def _run_search(self, legit_urls=[], event_shutdown=threading.Event()):
         while True:
             for header in self.headers:
-                # Force return results if True
+                # Force shutdown if True
                 if event_shutdown.is_set():
                     return legit_urls
                 r = requests.get(self.BASE_URL + urllib.parse.quote(self.search_query.replace(' ', '+')), headers=header)
@@ -173,12 +222,16 @@ class YoutubeSearch:
                         event_shutdown.set()
                         return legit_urls
 
+
     def _search(self, timeout=None):
         if timeout is None:
-            return self._run_search()
+            urls = []
+            event = threading.Event()
+            return self._run_search(legit_urls=urls, event_shutdown=event)
         else:
             wait_event = threading.Event()
             legit_urls = []
+            self._wait_event = wait_event
             worker = threading.Thread(target=self._run_search, name='worker_youtube_search_requests', args=(legit_urls, wait_event), daemon=True)
             worker.start()
             wait_event.wait(timeout)
